from __future__ import unicode_literals

import os

from mopidy import config, ext

__version__ = '0.2.6'

class Extension(ext.Extension):
    dist_name = 'Mopidy-AudioAddict'
    ext_name = 'audioaddict'
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(Extension, self).get_config_schema()
        schema['username'] = config.String(optional=True)
        schema['password'] = config.Secret(optional=True)
        schema['quality'] = config.String(optional=True)
        schema['difm'] = config.Boolean(optional=True)
        schema['radiotunes'] = config.Boolean(optional=True)
        schema['rockradio'] = config.Boolean(optional=True)
        schema['jazzradio'] = config.Boolean(optional=True)
        schema['frescaradio'] = config.Boolean(optional=True)
        return schema

    def setup(self, registry):
        from .actor import AudioAddictBackend
        registry.add('backend', AudioAddictBackend)



from __future__ import unicode_literals

import re

from mopidy.models import Ref

def unparse_uri(variant, identifier):
    return b'audioaddict:%s:%s' % (variant, identifier)


def parse_uri(uri):
    result = re.findall(r'^audioaddict:([a-z]+)(?::(\d+|[a-z]{2}))?$', uri)
    if result:
        return result[0]
    return None, None


def channel_to_ref(channel):
    name = channel.get('name', channel['streamurl']).strip()
    uri = unparse_uri('channel', channel['id'])
    return Ref.track(uri=uri, name='%s' % (name))


def radiostation_to_ref(radiostation):
    uri = unparse_uri('radiostation', radiostation['id'])
    return Ref.directory(uri=uri, name=radiostation.get('name', uri))




from __future__ import unicode_literals

import json
import logging
import time
import urlparse
import requests

logger = logging.getLogger(__name__)

class AudioAddict(object):

    proxies = None

    def __init__(self, username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy=None):
        self._cache = {}
        self._channels = {}
        self._backoff_until = time.time()
        self._backoff_max = 60
        self._backoff = 1
        self._username = username
        self._password = password
        self._quality = quality
        self._difm = difm
        self._radiotunes = radiotunes
        self._rockradio = rockradio
        self._jazzradio = jazzradio
        self._frescaradio = frescaradio

        # Set up proxy
        if proxy is not None:
            r1 = urlparse.urlsplit(proxy)
            self.proxies = {r1.scheme: proxy}

        # Figure out our API key
        if self._username and self._password:
            self._api_key, self._has_premium = self._fetchApiKey(self._username, self._password)
        else:
            self._api_key = None
            self._has_premium = False
            logger.info('AudioAddict: premium streams not available on your account')

    def flush(self):
        self._cache = {}
        self._channels = {}

    def radiostations(self, radiostation=None):
        stationlist = []
        if self._difm:
            stationlist.append({'id': 1, 'name': 'Digitally Imported', 'shortcode': 'difm'})
        if self._radiotunes:
            stationlist.append({'id': 2, 'name': 'RadioTunes', 'shortcode': 'radiotunes'})
        if self._rockradio:
            stationlist.append({'id': 3, 'name': 'RockRadio', 'shortcode': 'rockradio'})
        if self._jazzradio:
            stationlist.append({'id': 4, 'name': 'JazzRadio', 'shortcode': 'jazzradio'})
        if self._frescaradio:
            stationlist.append({'id': 5, 'name': 'FrescaRadio', 'shortcode': 'frescaradio'})

        if not stationlist:
            # Default to at least one network
            stationlist.append({'id': 1, 'name': 'Digitally Imported', 'shortcode': 'difm'})

        return stationlist

    def channels(self, radiostation=None):
        if radiostation:
            radiostation = int(radiostation)
            if radiostation == 1:
                station_name = 'Digitally Imported'
                hostname = 'listen.di.fm'
            elif radiostation == 2:
                station_name = 'RadioTunes'
                hostname = 'listen.radiotunes.com'
            elif radiostation == 3:
                station_name = 'RockRadio'
                hostname = 'listen.rockradio.com'
            elif radiostation == 4:
                station_name = 'JazzRadio'
                hostname = 'listen.jazzradio.com'
            elif radiostation == 5:
                station_name = 'FrescaRadio'
                hostname = 'listen.frescaradio.com'
            else:
                return []

            station_uri = 'http://' + hostname + '/streamlist'
            channels = self._fetch(station_uri, [])

            channel_count = len(channels)
            logger.info('AudioAddict: loading %s channels on %s', channel_count, station_name)
        else:
            return []

        if self._api_key and self._has_premium:
            if self._quality:
                if (self._quality == '320k'):
                    streampls = 'premium_high'
                if (self._quality == '128k'):
                    streampls = 'premium'
                if (self._quality == '64k'):
                    streampls = 'premium_medium'
                if (self._quality == '40k'):
                    streampls = 'premium_low'
            else:
                streampls = 'premium_high'
        else:
            streampls = 'public3'

        for channel in channels:
            if self._api_key:
                channel['streamurl'] = 'http://' + hostname + '/' + streampls + '/' + channel['key'] + '.pls?' + self._api_key
            else:
                channel['streamurl'] = 'http://' + hostname + '/' + streampls + '/' + channel['key'] + '.pls'

            self._channels.setdefault(channel['id'], channel)
        return channels

    def channel(self, identifier):
        identifier = int(identifier)
        if identifier in self._channels:
            return self._channels[identifier]
        path = '/id/%s' % identifier
        channel = self._fetch('channel', path, {})
        if channel:
            self._channels.setdefault(channel['id'], channel)
        return channel

    def _fetchApiKey(self, username, password):
        logger.info('AudioAddict: authenticating with API')

        payload = {'username': username, 'password': password}
        r = requests.post("https://api.audioaddict.com/v1/di/members/authenticate", data=payload, proxies=self.proxies)
        json_object_raw = r.json()

        subscriptions = json_object_raw['subscriptions']
        listen_key = json_object_raw['listen_key']

        if not subscriptions:
            has_premium = False
            logger.info('AudioAddict: premium streams not available on your account')
        else:
            has_premium = True
            logger.info('AudioAddict: premium streams enabled on your account')

        return (listen_key, has_premium)

    def _fetch(self, uri, default):
        if uri in self._cache:
            logger.debug('Cache hit: %s', uri)
            return self._cache[uri]

        if time.time() < self._backoff_until:
            logger.debug('Back off fallback used: %s', uri)
            return default

        logger.debug('Fetching: %s', uri)
        try:
            r = requests.get(uri, proxies=self.proxies)
            logger.debug("Get %s : %i", uri, r.status_code)

            if r.status_code is not 200:
                logger.error("AudioAddict: %s is not reachable [http code:%i]", uri, r.status_code)
                return None

        except requests.exceptions.RequestException, e:
            logger.error("AudioAddict RequestException: %s", e)
        except requests.exceptions.ConnectionError, e:
            logger.error("AudioAddict ConnectionError: %s", e)
        except requests.exceptions.URLRequired, e:
            logger.error("AudioAddict URLRequired: %s", e)
        except requests.exceptions.TooManyRedirects, e:
            logger.error("AudioAddict TooManyRedirects: %s", e)
        except Exception, e:
            logger.error("AudioAddict exception: %s", e)
        else:
            # Parse JSON object
            data = r.json()
            self._cache[uri] = data
            self._backoff = 1
            return data

        self._backoff = min(self._backoff_max, self._backoff*2)
        self._backoff_until = time.time() + self._backoff
        logger.debug('Entering back off mode for %d seconds.', self._backoff)
        return default



import unittest

class TestFormatProxy(unittest.TestCase):
    def test_format_proxy(self):
        # self.assertEqual(expected, format_proxy(scheme, username, password, hostname, port))
        assert False # TODO: implement your test here

class TestAudioAddictBackend(unittest.TestCase):
    def test___init__(self):
        # audio_addict_backend = AudioAddictBackend(config, audio)
        assert False # TODO: implement your test here

class TestAudioAddictLibrary(unittest.TestCase):
    def test_browse(self):
        # audio_addict_library = AudioAddictLibrary()
        # self.assertEqual(expected, audio_addict_library.browse(uri))
        assert False # TODO: implement your test here

    def test_find_exact(self):
        # audio_addict_library = AudioAddictLibrary()
        # self.assertEqual(expected, audio_addict_library.find_exact(query, uris))
        assert False # TODO: implement your test here

    def test_lookup(self):
        # audio_addict_library = AudioAddictLibrary()
        # self.assertEqual(expected, audio_addict_library.lookup(uri))
        assert False # TODO: implement your test here

    def test_refresh(self):
        # audio_addict_library = AudioAddictLibrary()
        # self.assertEqual(expected, audio_addict_library.refresh(uri))
        assert False # TODO: implement your test here

    def test_search(self):
        # audio_addict_library = AudioAddictLibrary()
        # self.assertEqual(expected, audio_addict_library.search(query, uris))
        assert False # TODO: implement your test here

class TestAudioAddictPlayback(unittest.TestCase):
    def test_change_track(self):
        # audio_addict_playback = AudioAddictPlayback()
        # self.assertEqual(expected, audio_addict_playback.change_track(track))
        assert False # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()



import unittest

class TestAudioAddict(unittest.TestCase):
    def test___init__(self):
        # audio_addict = AudioAddict(username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy)
        assert False # TODO: implement your test here

    def test_channel(self):
        # audio_addict = AudioAddict(username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy)
        # self.assertEqual(expected, audio_addict.channel(identifier))
        assert False # TODO: implement your test here

    def test_channels(self):
        # audio_addict = AudioAddict(username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy)
        # self.assertEqual(expected, audio_addict.channels(radiostation))
        assert False # TODO: implement your test here

    def test_flush(self):
        # audio_addict = AudioAddict(username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy)
        # self.assertEqual(expected, audio_addict.flush())
        assert False # TODO: implement your test here

    def test_radiostations(self):
        # audio_addict = AudioAddict(username, password, quality, difm, radiotunes, rockradio, jazzradio, frescaradio, proxy)
        # self.assertEqual(expected, audio_addict.radiostations(radiostation))
        assert False # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()



from __future__ import unicode_literals

import logging

import pykka

from mopidy import backend
from mopidy.models import Ref, Track

from . import client, translator

logger = logging.getLogger(__name__)

def format_proxy(scheme, username, password, hostname, port):
    # Format Proxy URL
    if hostname:
        # scheme must exists, so if None is give, we set default to http
        if not scheme:
            scheme = "http"
        # idem with port, default at 80
        if not port or port < 0:
            port = 80
        # with authentification
        if username and password:
            return "%s://%s:%s@%s:%i" % (
                scheme, username, password, hostname, port)
        # ... or without
        else:
            return "%s://%s:%i" % (scheme, hostname, port)
    else:
        return None

class AudioAddictBackend(pykka.ThreadingActor, backend.Backend):
    uri_schemes = ['audioaddict']

    def __init__(self, config, audio):
        super(AudioAddictBackend, self).__init__()

        full_proxy = format_proxy(
            scheme=config['proxy']['scheme'],
            username=config['proxy']['username'],
            password=config['proxy']['password'],
            hostname=config['proxy']['hostname'],
            port=config['proxy']['port'])

        self.audioaddict = client.AudioAddict(
            config['audioaddict']['username'],
            config['audioaddict']['password'],
            config['audioaddict']['quality'],
            config['audioaddict']['difm'],
            config['audioaddict']['radiotunes'],
            config['audioaddict']['rockradio'],
            config['audioaddict']['jazzradio'],
            config['audioaddict']['frescaradio'],
            proxy=full_proxy
        )
        self.library = AudioAddictLibrary(backend=self)
        self.playback = AudioAddictPlayback(audio=audio, backend=self)


class AudioAddictLibrary(backend.LibraryProvider):
    root_directory = Ref.directory(uri='audioaddict:root', name='AudioAddict')

    def browse(self, uri):
        result = []
        variant, identifier = translator.parse_uri(uri)

        if variant == 'root':
            for radiostation in self.backend.audioaddict.radiostations():
                result.append(translator.radiostation_to_ref(radiostation))
        elif variant == 'radiostation' and identifier:
            for channel in self.backend.audioaddict.channels(radiostation=identifier):
                result.append(translator.channel_to_ref(channel))
        else:
            logger.debug('Unknown URI: %s', uri)

        result.sort(key=lambda ref: ref.name)
        return result

    def refresh(self, uri=None):
        self.backend.audioaddict.flush()

    def lookup(self, uri):
        variant, identifier = translator.parse_uri(uri)
        if variant != 'channel':
            return []
        channel = self.backend.audioaddict.channel(identifier)
        if not channel:
            return []
        ref = translator.channel_to_ref(channel)

        return [Track(uri=ref.uri, name=ref.name)]

    def find_exact(self, query=None, uris=None):
        return None

    def search(self, query=None, uris=None):
        return None


class AudioAddictPlayback(backend.PlaybackProvider):
    def change_track(self, track):
        variant, identifier = translator.parse_uri(track.uri)
        if variant != 'channel':
            return False
        channel = self.backend.audioaddict.channel(identifier)
        track = track.copy(uri=channel['streamurl'])
        return super(AudioAddictPlayback, self).change_track(track)



import unittest

class TestUnparseUri(unittest.TestCase):
    def test_unparse_uri(self):
        # self.assertEqual(expected, unparse_uri(variant, identifier))
        assert False # TODO: implement your test here

class TestParseUri(unittest.TestCase):
    def test_parse_uri(self):
        # self.assertEqual(expected, parse_uri(uri))
        assert False # TODO: implement your test here

class TestChannelToRef(unittest.TestCase):
    def test_channel_to_ref(self):
        # self.assertEqual(expected, channel_to_ref(channel))
        assert False # TODO: implement your test here

class TestRadiostationToRef(unittest.TestCase):
    def test_radiostation_to_ref(self):
        # self.assertEqual(expected, radiostation_to_ref(radiostation))
        assert False # TODO: implement your test here

if __name__ == '__main__':
    unittest.main()
